@@ -7,29 +7,26 @@ for which a new license (GPL+exception) is in place.
 #ifndef SCFONTS_H
 #define SCFONTS_H
 
-#include <qstring.h>
-#include <qstrlist.h>
-#include <qstringlist.h>
-#include <qdict.h>
-//#include <qvector.h>
-#include <qfont.h>
-#include <qmap.h>
-#include <qdatetime.h>
+#include <QByteArray>
+#include <QDateTime>
+#include <QFont>
+#include <QList>
+#include <QMap>
+#include <QString>
+#include <QStringList>
+
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_OUTLINE_H
 #include FT_GLYPH_H
 
-
 FT_Error ftIOFunc( FT_Stream fts, unsigned long offset, unsigned char* buffer, unsigned long count);
 
-// #include <qpixmap.h>
-
-#include "scribusapi.h"
+#include "fonts/scface.h"
 #include "fpointarray.h"
 #include "scconfig.h"
+#include "scribusapi.h"
 
-#include "fonts/scface.h"
 
 class ScribusDoc;
 
@@ -52,7 +49,7 @@ class SCRIBUS_API SCFonts : public QMap<QString,ScFace>
 		/// Returns a font with that name; creates a replacement font if not found
 		const ScFace& findFont(const QString& fontName, ScribusDoc* doc = NULL);
 		/// Returns a map of pairs (scName, replacementName). Using this map for replaceFonts() will make substitutions permanent
-		QMap<QString,QString> getSubstitutions(const QValueList<QString> skip = QValueList<QString>()) const;
+		QMap<QString,QString> getSubstitutions(const QList<QString> skip = QList<QString>()) const;
 		/// Changes replacement fonts to point to new real fonts. For all keys 'nam' in 'substitutes', findFont(name).isReplacement() must be true
 		void setSubstitutions(const QMap<QString,QString>& substitutes, ScribusDoc* doc = NULL);
 		void removeFont(QString name);
@@ -67,12 +64,12 @@ class SCRIBUS_API SCFonts : public QMap<QString,ScFace>
 #ifdef HAVE_FONTCONFIG
 		void AddFontconfigFonts();
 #else
-#ifndef QT_MAC
+#ifndef Q_OS_MAC
 		void AddXFontServerPath();
 		void AddXFontPath();
 #endif
 #endif
-		QStrList FontPath;
+		QStringList FontPath;
 		QString ExtraPath;
 		struct testCache
 		{
