@@ -7,15 +7,14 @@ for which a new license (GPL+exception) is in place.
 #ifndef _SCPLUGIN_H
 #define _SCPLUGIN_H
 
-#include "qobject.h"
-#include "qwidget.h"
-#include "qstring.h"
-#include "qpixmap.h"
-#include "qdatetime.h"
-#include "qiconset.h"
+#include <QObject>
+#include <QWidget>
+#include <QString>
+#include <QPixmap>
+#include <QDateTime>
+#include <QIcon>
 #include "scribusapi.h"
 
-class QWidget;
 class ScribusDoc;
 class ScribusMainWindow;
 class DeferredTask;
@@ -197,6 +196,12 @@ class SCRIBUS_API ScPlugin : public QObject
 		 */
 		const QString & lastError() const;
 
+		//! \brief Returns if lastError message is not empty
+		bool hasLastError() const;
+
+		//! \brief Clear last error message
+		void clearLastError();
+
 		/*! @brief Update all user-visible text to reflect current UI language
 		 *
 		 * Updates the text on all widgets on the plug-in to reflect the new
@@ -215,6 +220,11 @@ class SCRIBUS_API ScPlugin : public QObject
 
 		//! \brief Allow plugins to add to a main menu
 		virtual void addToMainWindowMenu(ScribusMainWindow *) = 0;
+
+		//! \brief hooks that plugins can use to detect if the current document was closed etc..
+		virtual void setDoc(ScribusDoc* doc);
+		virtual void unsetDoc();
+		virtual void changedDoc(ScribusDoc* doc);
 	protected:
 		//! \brief Human readable, translated version of last error to occur.
 		QString m_lastError;
@@ -254,12 +264,36 @@ class SCRIBUS_API ScActionPlugin : public ScPlugin
 			QString keySequence;
 			QString menu;
 			QString menuAfterName;
-			QIconSet iconSet;
+			QString parentMenu;
+			QString subMenuName;
+			QString toolbar;			// Name of the ToolBar the action is to be inserted, if that toolbar doesn't exits it will be created
+			QString toolBarName;		// translateable ToolBar title
+			QPixmap icon1;
+			QPixmap icon2;
+			QList<int> notSuitableFor;		// a list of PageItem type values which the plugin can *not* handle
+			QList<int> forAppMode;			// a list of AppMode values for which the plugin will be active, an empty list indicates that the plugin is always active
+			int needsNumObjects;			// plugin needs this number of selected Objects. -1 = needs no Object, num > 2 any number of Objects is allowed
+			QList<int> firstObjectType;		// when needsNumObjects is 2 this list contains the Object Types of the first object on the selection
+			QList<int> secondObjectType;	// when needsNumObjects is 2 this list contains the Object Types of the second object on the selection
+											// -1 indicates that any kind of object is possible. Otherwise the selection must contain the 2 Object Types
+											// for the Plugin Action to be enabled
 			bool enabledOnStartup;
 		};
 
 		// Return an ActionInfo instance to the caller
 		const ActionInfo & actionInfo() const;
+		/**
+		* @brief handle the documents selection.
+		*
+		* This function is dedicated for handling the documents selection
+		* By default this uses the notSuitableFor, forAppMode, needsNumObjects,
+		* firstObjectType and secondObjectType variables.
+		* If you need more control over the selection reimplement this function.
+		*
+		* @returns bool true when the Action should be enabled.
+		*
+		*/
+		virtual bool handleSelection(ScribusDoc* doc, int SelectedType = -1);
 
 	public slots:
 		/**
@@ -461,7 +495,7 @@ class SCRIBUS_API ScPersistentPlugin : public ScPlugin
 //
 // The API version is currently simply incremented with each incompatible
 // change. Future versions may introduce a minor/major scheme if necessary.
-#define PLUGIN_API_VERSION 0x00000008
+#define PLUGIN_API_VERSION 0x00000009
 
 
 #endif
