@@ -30,37 +30,43 @@ for which a new license (GPL+exception) is in place.
 #include <vector>
 
 // include files for QT
-#include <qapplication.h>
-#include <qmainwindow.h>
-#include <qaction.h>
-#include <qmenubar.h>
-#include <qpopupmenu.h>
-#include <qtoolbar.h>
-#include <qtoolbutton.h>
-#include <qstatusbar.h>
-#include <qtooltip.h>
-#include <qstring.h>
-#include <qpixmap.h>
-#include <qmessagebox.h>
-#include <qpainter.h>
-#include <qmap.h>
-#include <qdict.h>
-#include <qguardedptr.h>
-#include <qfont.h>
-#include <qtimer.h>
-#include <qintdict.h>
-#include <qprogressdialog.h>
-#include <qprogressbar.h>
-#include <qworkspace.h>
-#include <qptrlist.h>
-#include <qclipboard.h>
-#include <qprocess.h>
+#include <QAction>
+#include <QActionGroup>
+#include <QApplication>
+#include <QClipboard>
+#include <QCloseEvent>
+#include <QDragEnterEvent>
+#include <QDropEvent>
+#include <QEvent>
+#include <QFont>
+#include <QKeyEvent>
+#include <QLabel>
+#include <QMainWindow>
+#include <QMap>
+#include <QMenu>
+#include <QMenuBar>
+#include <QMessageBox>
+#include <QMouseEvent>
+#include <QMultiHash>
+#include <QPainter>
+#include <QPixmap>
+#include <QPointer>
+#include <QProcess>
+#include <QProgressBar>
+#include <QProgressDialog>
+#include <QStatusBar>
+#include <QString>
+#include <QTimer>
+#include <QToolButton>
+#include <QToolTip>
+#include <QWheelEvent>
+#include <QWorkspace>
+
 
 // application specific includes
 #include "scribusapi.h"
 #include "scribusview.h"
 #include "scribusdoc.h"
-#include "scribusstructs.h"
 #include "customfdialog.h"
 
 class ActionManager;
@@ -68,42 +74,44 @@ class AlignDistributePalette;
 class Autoforms;
 class Biblio;
 class BookPalette;
-class CheckDocument;
-
-class FontCombo;
-class GuideManager;
 class CharSelect;
+class CheckDocument;
 class ColorCombo;
+class EditToolBar;
+class FileToolBar;
+class FontCombo;
+class FormatsManager;
+class GuideManager;
+class HelpBrowser;
 class LayerPalette;
-class LineFormate;
-class Mpalette;
 class Measurements;
 class MenuManager;
+class ModeToolBar;
 class NodePalette;
+class OutlinePalette;
+class PDFToolBar;
+class PSLib;
 class PageItem;
 class PagePalette;
 class Preferences;
-class PrefsManager;
 class PrefsContext;
-class PSLib;
+class PrefsManager;
+class PropertiesPalette;
 class ReformDoc;
+class ScToolBar;
 class ScrAction;
 class ScribusCore;
 class ScribusMainWindow;
 class ScribusQApp;
 class ScribusWin;
-class ScToolBar;
 class SimpleState;
-class StilFormate;
 class StoryEditor;
 class StyleManager;
 class TOCGenerator;
-class Tree;
 class UndoManager;
 class UndoPalette;
 class UndoState;
-class ModeToolBar;
-class PDFToolBar;
+
 
 extern SCRIBUS_API ScribusQApp* ScQApp;
 
@@ -126,29 +134,34 @@ public:
 	* \retval 0 - ok, 1 - no fonts, ...
 	*/
 	int initScMW(bool primaryMainwWindow);
+	void addScToolBar(ScToolBar *tb, QString name);
 	bool warningVersion(QWidget *parent);
 	void SetShortCut();
 	void startUpDialog();
 	void setDefaultPrinter(const QString&, const QString&, const QString&);
-	void getDefaultPrinter(QString*, QString*, QString*);
+	void getDefaultPrinter(QString& name, QString& file, QString& command);
 
-	ScribusDoc *doFileNew(double width, double height, double topMargin, double leftMargin, double rightMargin, double bottomMargin, double columnDistance, double columnCount, bool autoTextFrames, int pageArrangement, int unitIndex, int firstPageLocation, int orientation, int firstPageNumber, const QString& defaultPageSize, bool requiresGUI, int pageCount=1, bool showView=true);
-	ScribusDoc *newDoc(double width, double height, double topMargin, double leftMargin, double rightMargin, double bottomMargin, double columnDistance, double columnCount, bool autoTextFrames, int pageArrangement, int unitIndex, int firstPageLocation, int orientation, int firstPageNumber, const QString& defaultPageSize, bool requiresGUI, int pageCount=1, bool showView=true);
-	bool DoFileSave(QString fn);
+	inline bool scriptIsRunning(void) const { return (ScriptRunning > 0); }
+	inline void setScriptRunning(bool value) { ScriptRunning += (value ? 1 : -1); }
+
+	ScribusDoc *doFileNew(double width, double height, double topMargin, double leftMargin, double rightMargin, double bottomMargin, double columnDistance, double columnCount, bool autoTextFrames, int pageArrangement, int unitIndex, int firstPageLocation, int orientation, int firstPageNumber, const QString& defaultPageSize, bool requiresGUI, int pageCount=1, bool showView=true, int marginPreset=0);
+	ScribusDoc *newDoc(double width, double height, double topMargin, double leftMargin, double rightMargin, double bottomMargin, double columnDistance, double columnCount, bool autoTextFrames, int pageArrangement, int unitIndex, int firstPageLocation, int orientation, int firstPageNumber, const QString& defaultPageSize, bool requiresGUI, int pageCount=1, bool showView=true, int marginPreset=0);
+	bool DoFileSave(const QString& fileName, QString* savedFileName = NULL);
+	void changeEvent(QEvent *e);
 	void closeEvent(QCloseEvent *ce);
 	void keyPressEvent(QKeyEvent *k);
 	void keyReleaseEvent(QKeyEvent *k);
-	void mouseReleaseEvent(QMouseEvent *m);
-	void wheelEvent(QWheelEvent *w);
 	void setTBvals(PageItem *currItem);
-	void ShowSubs();
+	int ShowSubs();
 	void applyNewMaster(QString name);
 	void updateRecent(QString fn);
+	void doPasteRecent(QString data);
 	QString GetLang(QString inLang);
 	bool getPDFDriver(const QString & fn, const QString & nam,
 					  int Components, const std::vector<int> & pageNs,
-					  const QMap<int,QPixmap> & thumbs);
-	bool DoSaveAsEps(QString fn);
+					  const QMap<int,QPixmap> & thumbs, QString& error, 
+					  bool* cancelled = NULL);
+	bool DoSaveAsEps(QString fn, QString& error);
 	QString CFileDialog(QString wDir = ".", QString caption = "", QString filter = "", QString defNa = "",
 						int optionFlags = fdExistingFiles, bool *docom = 0, bool *doFont = 0, bool *doProfiles = 0);
 	/*! \brief Recalculate the colors after changing CMS settings.
@@ -157,7 +170,6 @@ public:
 	void recalcColors(QProgressBar *dia = 0);
 	void SwitchWin();
 	void RestoreBookMarks();
-//	void ReorgFonts();
 
 	void emergencySave();
 
@@ -178,10 +190,8 @@ public:
 	void restoreDeletePage(SimpleState *state, bool isUndo);
 	struct CopyPasteBuffer Buffer;
 	struct CopyContentsBuffer contentsBuffer;
-	QString Buffer2;
-	QString Buffer3;
-	bool BuFromApp;
-	int HaveDoc;
+	bool internalCopy;
+	int  HaveDoc;
 	PrefsContext* dirs;
 	/** \brief view is the main widget which represents your working area. The View
 	 * class should handle all events of the view widget.  It is kept empty so
@@ -202,9 +212,9 @@ public:
 	QLabel* mainWindowYPosDataLabel;
 	GuideManager *guidePalette;
 	CharSelect *charPalette;
-	Mpalette *propertiesPalette;
+	PropertiesPalette *propertiesPalette;
 	NodePalette *nodePalette;
-	Tree *outlinePalette;
+	OutlinePalette *outlinePalette;
 	Biblio *scrapbookPalette;
 	LayerPalette* layerPalette;
 	PagePalette *pagePalette;
@@ -215,26 +225,24 @@ public:
 	AlignDistributePalette *alignDistributePalette;
 	StoryEditor* storyEditor;
 	StoryEditor* CurrStED;
-	QMap<QString,QString> Sprachen;
+// 	QMap<QString,QString> Sprachen; // <language not translated, file name>
 	QWorkspace *wsp;
 	ScribusWin* ActWin;
 	QClipboard *ClipB;
 	QString LoadEnc;
-	bool ScriptRunning;
-	Autoforms* SCustom;
-	ModeToolBar* mainToolBar;
-
 	QMap<QString, QStringList> InstLang;
 	QMap<QString,QString> LangTransl;
 
 	QProcess *ExternalApp;
 
-	QMap<QString, QGuardedPtr<ScrAction> > scrActions;
-	QMap<QString, QGuardedPtr<ScrAction> > scrRecentFileActions;
-	QMap<QString, QGuardedPtr<ScrAction> > scrWindowsActions;
-	QMap<QString, QGuardedPtr<ScrAction> > scrLayersActions;
-	QMap<QString, QGuardedPtr<ScrAction> > scrRecentPasteActions;
-	QDict<QActionGroup> scrActionGroups;
+	QMap<QString, QPointer<ScrAction> > scrActions;
+	QMap<QString, QPointer<ScrAction> > scrRecentFileActions;
+	QMap<QString, QPointer<ScrAction> > scrWindowsActions;
+	QMap<QString, QPointer<ScrAction> > scrScrapActions;
+	QMap<QString, QPointer<ScrAction> > scrLayersActions;
+	QMap<QString, QPointer<ScrAction> > scrRecentPasteActions;
+	QMap<QString, QPointer<ScToolBar> > scrToolBars;
+	QMultiHash<QString, QActionGroup*> scrActionGroups;
 	MenuManager* scrMenuMgr;
 	ActionManager* actionManager;
 	QStringList RecentDocs;
@@ -257,11 +265,13 @@ public slots:
 	//void setItemFillTransparency(double t);
 	//void setItemLineTransparency(double t);
 	void setStatusBarMousePosition(double xp, double yp);
+	void setStatusBarTextPosition(double base, double yp);
 	void setStatusBarInfoText(QString newText);
 	bool DoFileClose();
 	//bool DoSaveClose();
-	void fontMenuAboutToShow();
 	void windowsMenuAboutToShow();
+	//! \brief Handle the Extras menu for its items availability.
+	void extrasMenuAboutToShow();
 	void newActWin(QWidget *w);
 	void closeActiveWindowMasterPageEditor();
 	void updateActiveWindowCaption(const QString &newCaption);
@@ -272,7 +282,7 @@ public slots:
 	//void doDeHyphenate();
 	void slotTest();
 	void slotTest2();
-	void PutScrap();
+	void PutScrap(int scID);
 	void PutToPatterns();
 	void changeLayer(int);
 	void showLayer();
@@ -282,22 +292,27 @@ public slots:
 	void manageMasterPagesEnd();
 	/** \brief generate a new document in the current view */
 	bool slotFileNew();
+	void newFileFromTemplate();
 	bool slotPageImport();
 	bool loadPage(QString fileName, int Nr, bool Mpa, const QString& renamedPageName=QString::null);
 
 	void slotGetContent();
 	void slotGetContent2(); // kk2006
+	void slotGetClipboardImage();
 	/*!
 	\author Franz Schmid
 	\brief Appends a Textfile to the Text in the selected Textframe at the Cursorposition
 	*/
 	void slotFileAppend();
 
-	void removeRecent(QString fn);
+	void removeRecent(QString fn, bool fromFileWatcher = false);
+	void removeRecentFromWatcher(QString fn);
 	void loadRecent(QString fn);
 	void rebuildRecentFileMenu();
 	void rebuildRecentPasteMenu();
 	void pasteRecent(QString fn);
+	void pasteFromScrapbook(QString fn);
+	void importVectorFile();
 	void rebuildLayersList();
 	bool slotDocOpen();
 	bool loadDoc(QString);
@@ -322,9 +337,10 @@ public slots:
 	\author Franz Schmid
 	\brief Generate and print PostScript from a doc
 	\param options PrintOptions struct to control all settings
+	\param error   Error Message in case of failure
 	\sa ScribusMainWindow::slotFilePrint()
 	\retval bool True for success */
-	bool doPrint(PrintOptions &options);
+	bool doPrint(PrintOptions &options, QString& error);
 	/** \brief exits the application */
 	void slotFileQuit();
 	/** \brief put the marked text/object into the clipboard and remove
@@ -338,16 +354,21 @@ public slots:
 	void slotEditPasteContents(int absolute=0);
 	void EnableTxEdit();
 	void DisableTxEdit();
-	void SelectAll();
+	void SelectAll(bool docWideSelect=false);
+	void SelectAllOnLayer();
 	void deselectAll();
 	void ClipChange();
 	//void clearContents();
 	/** \brief shows an about dialog*/
 	void slotHelpAbout();
 	void slotHelpAboutPlugins();
-    void slotHelpAboutQt();
-	void slotOnlineHelp();
+	void slotHelpAboutQt();
+	void slotHelpCheckUpdates();
+	void slotRaiseOnlineHelp();
+	void slotOnlineHelp(const QString & jumpToSection=QString::null, const QString & jumpToFile=QString::null);
+	void slotOnlineHelpClosed();
 	void ToggleTips();
+	void ToggleMouseTips();
 	/** \brief Erzeugt eine neue Seite */
 	void slotNewPageP(int wo, QString templ);
 	void slotNewPageM();
@@ -355,17 +376,17 @@ public slots:
 	void slotNewPage(int w, const QString& masterPageName=QString::null, bool mov = true);
 	void duplicateToMasterPage();
 	/** \brief Loescht die aktuelle Seite */
-	void DeletePage();
+	void deletePage();
 	/**
 	 * \brief Delete pages
 	 * @param from First page to delete
 	 * @param to Last page to delete
 	 */
-	void DeletePage(int from, int to);
-	void DeletePage2(int pg);
+	void deletePage(int from, int to);
+	void deletePage2(int pg);
 	/** \brief Verschiebt Seiten */
-	void MovePage();
-	void CopyPage();
+	void movePage();
+	void copyPage();
 	void changePageMargins();
 	/*!
 	\author Craig Bradney
@@ -404,7 +425,7 @@ public slots:
 	/** \brief Schaltet Rasterbenutzung ein/aus */
 	void ToggleURaster();
 	/** \brief Schaltet Rahmenbearbeitung ein/aus */
-	void ToggleFrameEdit();
+	void toggleNodeEdit();
 	void slotSelect();
 	/** \brief Switch appMode
 	\param mode TODO learn modes*/
@@ -413,26 +434,20 @@ public slots:
 	/** \brief Neues Dokument erzeugt */
 	void HaveNewDoc();
 	void HaveNewSel(int Nr);
-	void rebuildStyleMenu(int itemType);
 	/** Dokument ist geaendert worden */
 	void slotDocCh(bool reb = true);
-	/** Setzt die Farbe */
-	void setItemFarbe(int id);
 	/** Setzt die Abstufung */
 	void setItemShade(int id);
 	/** Setzt den Font */
-	void setItemFont(int id);
-	void setItemFont2(int id);
 	/** Korrigiert das FontMenu */
-	void AdjustFontMenu(const QString& nf);
 	void SetNewFont(const QString& nf);
 	/** Setz die Zeichensatzgroesse */
 	void setItemFSize(int id);
-	void setFSizeMenu(int size);
 	/** Farbeditor */
 	void slotEditColors();
+	/** Color Replacement */
+	void slotReplaceColors();
 	/** Style Manager */
-// 	void slotStyleManager();
 	/*
 	// Setzt den Pen-Tonwert
 	void setPenShade(int sh);
@@ -447,28 +462,27 @@ public slots:
 	// Setzt die Brush-Farbe
 	void setBrushFarbe(QString farbe);
 	*/
-	void setCSMenu();
+	//CB: void setCSMenu();
 	/** Fragt nach den Farben */
 //	void GetBrushPen();
 	/** Erzeugt einen Rahmen */
 	void MakeFrame(int f, int c, double *vals);
 	//** Loescht ein Element */
 	//void DeleteObjekt();
-	/** Dupliziert das Element */
-	void ObjektDup();
-	/** Dupliziert das Element mehrfach*/
-	void ObjektDupM();
+	/** Duplicate current item */
+	void duplicateItem();
+	/** Multiple duplicate current item*/
+	void duplicateItemMulti();
 	/** \brief Refromat the document when user click "OK" in ReformDoc dialog.
 	See docSetup() for more info. */
 	bool slotDocSetup();
 	void objectAttributes();
 	void getImageInfo();
 	void generateTableOfContents();
-	void buildFontMenu();
 	/*! \brief Change Preferences dialog.
 	See prefsOrg for more info. It's very similar to docSetup/slotDocSetup. */
 	void slotPrefsOrg();
-	void saveStyles(StilFormate *dia); //still required for style save from SE
+//	void saveStyles(StilFormate *dia); //still required for style save from SE
 	void setNewAlignment(int a);
 	void setNewParStyle(const QString& name);
 	void setNewCharStyle(const QString& name);
@@ -488,7 +502,6 @@ public slots:
 	void setItemTypeStyle(int id);
 	void slotElemRead(QString Name, double x, double y, bool art, bool loca, ScribusDoc* docc, ScribusView* vie);
 	void slotChangeUnit(int art, bool draw = true);
-	void NoFrameEdit();
 	/*!
 	 * @brief Apply master pages from the Apply Master Page dialog
 	 * @todo Make this work with real page numbers, negative numbers and document sections when they are implemented
@@ -502,10 +515,9 @@ public slots:
 	void ToggleGuides();
 	void ToggleBase();
 	void ToggleUGuides();
-	void HaveRaster(bool art);
 	void EditTabs();
 	void SearchText();
-	void imageEditorExited();
+	void imageEditorExited(int, QProcess::ExitStatus);
 	/*! \brief call gimp and wait upon completion */
 	void callImageEditor();
 	void docCheckToggle(bool visible);
@@ -516,6 +528,7 @@ public slots:
 	void insertSampleText();
 	//void sendToLayer(int layerNumber);
 	void updateItemLayerList();
+	void updateColorLists();
 	/*! \brief Apply changes from ReformDoc dialog.
 	It's called from this->slotDocSetup() or from ReformDoc directly.
 	\param dia a reference to the ReformDoc dialog */
@@ -529,25 +542,27 @@ public slots:
 	//! \brief manages the documents patterns
 	void managePatterns();
 	//! \brief enable or disable the unicode actions and their menus
-	void enableTextActions(QMap<QString, QGuardedPtr<ScrAction> > *actionMap, bool enabled, const QString& fontName=QString::null);
+	void enableTextActions(QMap<QString, QPointer<ScrAction> > *actionMap, bool enabled, const QString& fontName=QString::null);
 	//! \brief allow SE to get the SM for edit stlyes
 	StyleManager *styleMgr() const {return styleManager;};
+	//! \brief drawnew, call palettes to update for new page layout
+	void updateGUIAfterPagesChanged();
 
 signals:
 	void TextStyle(const ParagraphStyle&);
 //deprecated: (av)
-	void TextISize(int);
+	void TextISize(double);
 	void TextIFont(const QString&);
-	void TextUSval(int);
+	void TextUSval(double);
 	void TextStil(int);
-	void TextFarben(QString, QString, int, int);
-	void TextScale(int);
-	void TextScaleV(int);
-	void TextBase(int);
-	void TextShadow(int, int);
-	void TextOutline(int);
-	void TextUnderline(int, int);
-	void TextStrike(int, int);
+	void TextFarben(QString, QString, double, double);
+	void TextScale(double);
+	void TextScaleV(double);
+	void TextBase(double);
+	void TextShadow(double, double);
+	void TextOutline(double);
+	void TextUnderline(double, double);
+	void TextStrike(double, double);
 
 protected:
 	/*!
@@ -572,20 +587,19 @@ private:
 	void initScrapbook();
 
 	void updateColorMenu(QProgressBar* progressBar=NULL);
+	void ToggleFrameEdit();
+	void NoFrameEdit();
+
+	int ScriptRunning;
 
 	QLabel* mainWindowStatusLabel;
 	QString recentFileMenuName;
 	QString recentPasteMenuName;
 	QString layerMenuName;
 	QPixmap noIcon;
-	ColorCombo *ColorMenC;
-	/** ShapeMenu enthaelt die Rahmenformen */
-	QPopupMenu *ShapeMenu;
-	/** FontMenu enthaelt die Fonts */
-	QPopupMenu *FontMenu;
-	FontCombo* FontSub;
-	ScToolBar *fileToolBar;
-	ScToolBar *editToolBar;
+	EditToolBar *editToolBar;
+	FileToolBar *fileToolBar;
+	ModeToolBar* modeToolBar;
 	PDFToolBar* pdfToolBar;
 	QToolButton* DatOpe;
 	QToolButton* DatSav;
@@ -615,7 +629,6 @@ private:
 	bool objectSpecificUndo;
 
 	void addNewPages(int wo, int where, int numPages, double height, double width, int orient, QString siz, bool mov, QStringList* basedOn = 0);
-	QMap<int,QString> FontID;
 
 	void *PSDriver;
 	int DocNr;
@@ -624,7 +637,6 @@ private:
 					QString Pname;
 					QString Dname;
 					QString Command;
-					QByteArray DevMode;
 				} PDef ;
 	TOCGenerator *tocGenerator;
 	int storedPageNum;
@@ -634,7 +646,9 @@ private:
 	StyleManager *styleManager;
 	UndoManager *undoManager;
 	PrefsManager *prefsManager;
-	QString currentFontForFontMenu;
+	FormatsManager *formatsManager;
+
+	QPointer<HelpBrowser> helpBrowser;
 };
 
 #endif
