@@ -29,9 +29,11 @@ for which a new license (GPL+exception) is in place.
 
 #include <vector>
 #include <utility>
-#include <qobject.h>
+#include <QObject>
+#include <QPixmap>
 
 #include "scribusapi.h"
+#include "transaction.h"
 #include "undostate.h"
 #include "undoobject.h"
 #include "undostack.h"
@@ -41,9 +43,23 @@ class QPixmap;
 class UndoGui;
 class PrefsContext;
 
+struct TransactionData;
+
 /** @brief Key is the doc name, value is it's undo stack */
 typedef QMap<QString, UndoStack> StackMap;
 
+class SCRIBUS_API TransactionSettings
+{
+public:
+	QString  targetName;
+	QPixmap *targetPixmap;
+	QString  actionName;
+	QString  description;
+	QPixmap *actionPixmap;
+
+	TransactionSettings(void) { targetPixmap = actionPixmap = NULL; }  
+};
+
 /**
  * @brief TransactionState provides a container where multiple UndoStates can be stored
  * @brief as a single action which then appears in the attached <code>UndoGui</code>s.
@@ -57,6 +73,7 @@ public:
 	TransactionState();
 	/** @brief Destroys the TransactionState instance */
 	~TransactionState();
+	
 	/**
 	 * @brief Add a new <code>UndoState</code> object to the transaction.
 	 * @param state state to be added to the transaction
@@ -114,6 +131,26 @@ private:
 	std::vector<UndoState*> states_;
 };
 
+
+
+/**
+    Class which handles Undo transactions. No data, just methods.
+ */
+class UndoTransaction : public Transaction
+{
+public:
+	UndoTransaction(TransactionData* data);
+	virtual ~UndoTransaction();
+	virtual bool commit();
+	virtual bool cancel();
+	bool commit(const QString &targetName,
+				QPixmap *targetPixmap,
+				const QString &name = "",
+				const QString &description = "",
+				QPixmap *actionPixmap = 0);
+};
+
+
 /**
  * @brief UndoManager handles the undo stack.
  *
@@ -135,6 +172,8 @@ class SCRIBUS_API UndoManager : public QObject
 {
 	Q_OBJECT
 public:
+	friend class UndoTransaction;
+	
 	/** @brief Marks for a global undo mode where ever UndoOjbect id is requested. */
 	static const int GLOBAL_UNDO_MODE = -1;
 	
@@ -184,25 +223,31 @@ public:
 	 * @param targetPixmap Icon for the target on which this transaction works.
 	 * this icon will be drawn first when the action is presented in Action History
 	 * window and icon for the action will be drawn over this one.
-	 * @param name name for the transaction (f.e. "Move" would make with the above
+	 * @param actionName name for the transaction (f.e. "Move" would make with the above
 	 * "Move Selection")
+	 * The result can only be used for initializing a Transaction object, eg. on the stack:
+	 *      Transaction groupTransaction(undoManger->beginTransaction(...));
+	 * or the heap:
+	 *      Transaction* groupTransactionPtr = new Transaction(undoManger->beginTransaction(...));
 	 * @param description description for the transaction
 	 * @param actionPixmap icon for the action performed by the transaction
 	 * @sa commit()
 	 */
-	void beginTransaction(const QString &targetName = "",
-	                      QPixmap *targetPixmap = 0,
-	                      const QString &actionName = "",
-	                      const QString &description = "",
-	                      QPixmap *actionPixmap = 0);
+	UndoTransaction beginTransaction(const QString &targetName = "",
+									 QPixmap *targetPixmap = 0,
+									 const QString &actionName = "",
+									 const QString &description = "",
+									 QPixmap *actionPixmap = 0);
+
+	UndoTransaction beginTransaction(const TransactionSettings& settings);
 	
 	/**
 	 * @brief Cancels the current transaction and deletes groupped <code>UndoState</code>s.
 	 * @brief Nothing from canceled transaction will be sent to the undo gui widgets.
 	 */
-	void cancelTransaction();
+	//void cancelTransaction();
 	
-	/**
+	/*
 	 * @brief Commit the current transaction.
 	 *
 	 * Current transaction will be commited and <code>UndoManager</code> will be switched
@@ -218,15 +263,15 @@ public:
 	 * @param actionPixmap icon for the action performed by the transaction
 	 * @sa beginTransaction()
 	 */
-	void commit(const QString &targetName = "",
-	            QPixmap *targetPixmap = 0,
-	            const QString &name = "",
-	            const QString &description = "",
-	            QPixmap *actionPixmap = 0);
+//	void commit(const QString &targetName = "",
+//	            QPixmap *targetPixmap = 0,
+//	            const QString &name = "",
+//	            const QString &description = "",
+//	            QPixmap *actionPixmap = 0);
 	
 	/**
 	 * @brief Returns true if in transaction mode if not will return false.
-	 * @return true if in transaction mode if not will return false
+	 * @return bool true if in transaction mode if not will return false
 	 */
 	bool isTransactionMode();
 	
@@ -309,25 +354,6 @@ public:
 	bool isGlobalMode();
 
 private:
-
-/*** UndoManager::TransactionObject ***************************************************/
-
-	/**
-	 * @brief Dummy subclass of <code>UndoObject</code> which is used for holding the name
-	 * @brief for the transaction's target object(group) (f.e "Selection", "Group", "Script"...).
-	 * @author Riku Leino tsoots@gmail.com
-	 * @date January 2005
-	 */
-	class TransactionObject : public UndoObject
-	{
-	public:
-		TransactionObject() {};
-		virtual ~TransactionObject() {};
-		void restore(UndoState*, bool) {};
-	};
-
-/**************************************************************************************/
-
 	/**
 	 * @brief The only instance of UndoManager available.
 	 *
@@ -365,19 +391,7 @@ private:
 	 * @brief Stores the transactions which are currently started but not
 	 * @brief canceled or commited.
 	 */
-	std::vector<std::pair<TransactionObject*, TransactionState*> > transactions_;
-
-	/**
-	 * @brief If in transaction mode this is the container for incoming <code>UndoState</code>
-	 * @brief objects.
-	 *
-	 * It is also used to detect if we are in the transaction mode. When it is <code>NULL</code>
-	 * normal mode is on.
-	 */
-	TransactionState *transaction_;
-
-	/** @brief Dummy object for storing transaction target's name */
-	TransactionObject *transactionTarget_;
+	std::vector<TransactionData*> transactions_;
 
 	/**
 	 * @brief UndoGuis attached to this UndoManager
@@ -417,10 +431,13 @@ private:
 	 */
 	void initIcons();
 
-	void setMenuTexts();
+	void setTexts();
 
 public:
-
+	/**
+	 * @brief Returns last saved Undo state
+	 */
+	UndoState* getLastUndoState();
 	/**
 	* @name Action strings
 	* Strings describing undo actions
@@ -454,6 +471,8 @@ public:
 	static QString CreateTo;
 	static QString AlignDistribute;
 	static QString ItemsInvolved;
+	static QString ItemsInvolved2;
+	static uint    ItemsInvolvedLimit;
 	static QString Cancel;
 	static QString SetFill;
 	static QString ColorFromTo;
@@ -505,6 +524,7 @@ public:
 	static QString SetFontEffect;
 	static QString ImageFrame;
 	static QString TextFrame;
+	static QString LatexFrame;
 	static QString Polygon;
 	static QString BezierCurve;
 	static QString Polyline;
@@ -512,7 +532,11 @@ public:
 	static QString ConvertTo;
 	static QString ImportSVG;
 	static QString ImportEPS;
+	static QString ImportBarcode;
 	static QString ImportOOoDraw;
+	static QString ImportAI;
+	static QString ImportXfig;
+	static QString ImportWMF;
 	static QString ScratchSpace;
 	//static QString TextFlow;
 	static QString ObjectFrame;
@@ -531,12 +555,15 @@ public:
 	static QString BreakRatio;
 	static QString EditContourLine;
 	static QString EditShape;
+	static QString ChangeShapeType;
 	static QString ResetContourLine;
 	static QString AddPage;
 	static QString AddPages;
 	static QString DeletePage;
 	static QString DeletePages;
+	static QString ChangePageProps;
 	static QString AddLayer;
+	static QString DuplicateLayer;
 	static QString DeleteLayer;
 	static QString RenameLayer;
 	static QString RaiseLayer;
@@ -545,7 +572,14 @@ public:
 	static QString PrintLayer;
 	static QString DoNotPrintLayer;
 	static QString SetLayerName;
+	static QString FlowLayer;
+	static QString DisableFlowLayer;
+	static QString SetLayerBlendMode;
+	static QString SetLayerTransparency;
+	static QString SetLayerLocked;
+	static QString SetLayerUnlocked;
 	static QString GetImage;
+	static QString ChangeFormula;
 	static QString MultipleDuplicate;
 	static QString ApplyTextStyle;
 	static QString MenuUndo;
@@ -561,6 +595,13 @@ public:
 	static QString Copy;
 	static QString CopyPage;
 	static QString ToOutlines;
+	static QString EditText;
+	static QString ClearImage;
+	static QString ClearContent;
+	static QString LinkTextFrame;
+	static QString UnlinkTextFrame;
+	static QString MovePage;
+	static QString PathOperation;
 	/*@}*/
 
 	/**
@@ -571,6 +612,7 @@ public:
 /*** Icons for UndoObjects *******************************************/
 	static QPixmap *IImageFrame;
 	static QPixmap *ITextFrame;
+	static QPixmap *ILatexFrame;
 	static QPixmap *ILine;
 	static QPixmap *IPolygon;
 	static QPixmap *IPolyline;
@@ -603,6 +645,9 @@ public:
 	static QPixmap *IFont;
 	static QPixmap *ISVG;
 	static QPixmap *IEPS;
+	static QPixmap *IAI;
+	static QPixmap *IXFIG;
+	static QPixmap *IWMF;
 	static QPixmap *IImportOOoDraw;
 	static QPixmap *IImageScaling;
 	static QPixmap *IBorder;
@@ -612,6 +657,7 @@ public:
 	static QPixmap *IDown;
 	static QPixmap *IPrint;
 	static QPixmap *IGetImage;
+	static QPixmap *IChangeFormula;
 	static QPixmap *IMultipleDuplicate;
 	/*@}*/
 
@@ -734,6 +780,14 @@ signals:
 	 */
 	void popBack();
 
+	/**
+	 * @brief This signal is emitted when beginning a series of undo/redo actions
+	 *
+	 * It could be used in the application as a signal to disable temporarily gui
+	 * updates.
+	 */
+	void undoRedoBegin();
+
 	/**
 	 * @brief This signal is emitted when all requested undo/redo actions have been done.
 	 *
